/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Jan Varga <varga@utcru.sk> (Original Author)
 */

#include "nsIServiceManager.h"
#include "nsIAutoCompleteResults.h"
#include "nsXPIDLString.h"

#include "nsDatabaseAutoCompleteSession.h"


nsDatabaseAutoCompleteSession::nsDatabaseAutoCompleteSession(void)
{
  NS_INIT_ISUPPORTS();
#ifdef DEBUG
  printf("nsDatabaseAutoCompleteSession::nsDatabaseAutoCompleteSession\n");
#endif
}

nsDatabaseAutoCompleteSession::~nsDatabaseAutoCompleteSession (void)
{
#ifdef DEBUG
  printf("nsDatabaseAutoCompleteSession::~nsDatabaseAutoCompleteSession\n");
#endif
}


NS_IMPL_ISUPPORTS2(nsDatabaseAutoCompleteSession,
                   nsIAutoCompleteSession,
                   nsIDatabaseAutoCompleteSession);


NS_IMETHODIMP
nsDatabaseAutoCompleteSession::OnStartLookup(const PRUnichar* aSearchString,
                                             nsIAutoCompleteResults* aPreviousSearchResult,
                                             nsIAutoCompleteListener* aListener)
{
  if (aSearchString[0] == 0) {
    aListener->OnAutoComplete(nsnull, nsIAutoCompleteStatus::ignored);
    return NS_OK;
  }

  nsCOMPtr<nsIAutoCompleteResults> results;
  results = do_CreateInstance(NS_AUTOCOMPLETERESULTS_CONTRACTID);

  results->SetDefaultItemIndex(-1);
  results->SetSearchString(aSearchString);

  nsCOMPtr<nsISupportsArray> array;
  results->GetItems(getter_AddRefs(array));

  AutoCompleteStatus status = nsIAutoCompleteStatus::noMatch;

  if (mProvider) {
    nsCOMPtr<nsIDatabaseRowList> rowList;
    mProvider->GetSearchResult(aSearchString, getter_AddRefs(rowList));
    if (rowList) {
      PRInt32 count;
      rowList->GetRowCount(&count);
      if (count) {
        results->SetDefaultItemIndex(0);
        status = nsIAutoCompleteStatus::matchFound;
        for (PRInt32 i = 0; i < count; i++) {
          nsCOMPtr<nsIAutoCompleteItem> newItem(do_CreateInstance(NS_AUTOCOMPLETEITEM_CONTRACTID));
          nsCOMPtr<nsIDatabaseRow> row;
          rowList->GetRow(i, getter_AddRefs(row));
          const PRUnichar* value;
          row->GetValueConst(0, &value);
          newItem->SetValue(value);
          array->AppendElement((nsISupports*)newItem);
        }
      }
    }
    else
      status = nsIAutoCompleteStatus::failed;
  }

  aListener->OnAutoComplete(results, status);

  return NS_OK;
}

NS_IMETHODIMP
nsDatabaseAutoCompleteSession::OnStopLookup() 
{
  return NS_OK;
}

NS_IMETHODIMP
nsDatabaseAutoCompleteSession::OnAutoComplete(const PRUnichar* aSearchString,
                                              nsIAutoCompleteResults* aPreviousSearchResult,
                                              nsIAutoCompleteListener* aListener)
{
  return NS_OK;
}


NS_IMETHODIMP
nsDatabaseAutoCompleteSession::GetProvider(nsIDatabaseAutoCompleteProvider** aProvider)
{
  *aProvider = mProvider;
  NS_IF_ADDREF(*aProvider);

  return NS_OK;
}

NS_IMETHODIMP
nsDatabaseAutoCompleteSession::SetProvider(nsIDatabaseAutoCompleteProvider* aProvider)
{
  mProvider = aProvider;

  return NS_OK;
}
