/*    
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Jan Varga <varga@utcru.sk> (Original Author)
 */

#include "nsReadableUtils.h"

#include "nsDatabaseOutlinerView.h"

#define DEBUG 1

nsDatabaseOutlinerView::nsDatabaseOutlinerView(void)
{
  NS_INIT_ISUPPORTS();
#ifdef DEBUG
  printf("nsDatabaseOutlinerView::nsDatabaseOutlinerView\n");
#endif
}

nsDatabaseOutlinerView::~nsDatabaseOutlinerView (void)
{
#ifdef DEBUG
  printf("nsDatabaseOutlinerView::~nsDatabaseOutlinerView\n");
#endif
}


// a circular strong reference
NS_IMPL_ADDREF(nsDatabaseOutlinerView)

NS_IMPL_QUERY_INTERFACE3(nsDatabaseOutlinerView,
                         nsIOutlinerView,
                         nsIDatabaseOutlinerView,
                         nsIDatabaseRowListObserver);

NS_IMETHODIMP_(nsrefcnt)
nsDatabaseOutlinerView::Release()
{
  NS_PRECONDITION(PRInt32(mRefCnt) > 0, "duplicate release");
  --mRefCnt;
  NS_LOG_RELEASE(this, mRefCnt, "nsDatabaseOutlinerView");

  // delete if the last reference is our strong circular reference
  if (mRowList && PRInt32(mRefCnt) == 1) {
    mRowList->RemoveObserver(this);
    return 0;
  }
  else if (mRefCnt == 0) {
    delete this;
    return 0;
  }
  return mRefCnt;
}

/*
NS_IMPL_ISUPPORTS3(nsDatabaseOutlinerView,
                   nsIOutlinerView,
                   nsIDatabaseOutlinerView,
                   nsIDatabaseRowListObserver);
*/


NS_IMETHODIMP nsDatabaseOutlinerView::GetRowCount(PRInt32* aRowCount)
{
  if (mRowList)
    mRowList->GetRowCount(aRowCount);
  else
    *aRowCount = 0;

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::GetSelection(nsIOutlinerSelection** aSelection)
{
  *aSelection = mSelection;
  NS_IF_ADDREF(*aSelection);

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::SetSelection(nsIOutlinerSelection* aSelection)
{
  mSelection = aSelection;

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::GetRowProperties(PRInt32 aRowIndex, nsISupportsArray* aProperties)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::GetCellProperties(PRInt32 aRowIndex, const PRUnichar* aColID, nsISupportsArray* aProperties)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::GetColumnProperties(const PRUnichar* aColID, nsIDOMElement* aColElt, nsISupportsArray* aProperties)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::IsContainer(PRInt32 aRowIndex, PRBool* _retval)
{
  *_retval = PR_FALSE;

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::IsContainerOpen(PRInt32 aRowIndex, PRBool* _retval)
{
  *_retval = PR_FALSE;

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::IsContainerEmpty(PRInt32 aRowIndex, PRBool* _retval)
{
  *_retval = PR_TRUE;

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::GetParentIndex(PRInt32 aRowIndex, PRInt32* _retval)
{
  *_retval = -1;

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::HasNextSibling(PRInt32 aRowIndex, PRInt32 aAfterIndex, PRBool* _retval)
{
  *_retval = PR_FALSE;

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::GetLevel(PRInt32 aRowIndex, PRInt32* _retval)
{
  *_retval = 0;
  
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::GetCellText(PRInt32 aRowIndex, const PRUnichar* aColID, PRUnichar** _retval)
{
  *_retval = nsnull;

  if (mRowList && aRowIndex >=0) {
    PRInt32 rowCount;
    mRowList->GetRowCount(&rowCount);
    if (aRowIndex < rowCount) {
      PRInt16 columnIndex;
      mRowList->GetColumnIndex(aColID, &columnIndex);
    
      if (columnIndex >= 0) {
        nsCOMPtr<nsIDatabaseRow> row;
        mRowList->GetRow(aRowIndex, getter_AddRefs(row));
        row->GetValue(columnIndex, _retval);
      }
    }
  }

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::SetOutliner(nsIOutlinerBoxObject* aBoxObject)
{
  mBoxObject = aBoxObject;
  
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::ToggleOpenState(PRInt32 aRowIndex)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::CycleHeader(const PRUnichar* aColID, nsIDOMElement *aColElt)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::SelectionChanged()
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::CycleCell(PRInt32 aRowIndex, const PRUnichar* aColID)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::IsEditable(PRInt32 aRowIndex, const PRUnichar* aColID, PRBool* _retval)
{
  *_retval = PR_FALSE;

  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::SetCellText(PRInt32 aRowIndex, const PRUnichar* aColID, const PRUnichar* aValue)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::PerformAction(const PRUnichar* aAction)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::PerformActionOnRow(const PRUnichar* aAction, PRInt32 aRowIndex)
{
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseOutlinerView::PerformActionOnCell(const PRUnichar* aAction, PRInt32 aRowIndex, const PRUnichar* aColID)
{
  return NS_OK;
}


NS_IMETHODIMP
nsDatabaseOutlinerView::GetRowList(nsIDatabaseRowList** aRowList)
{
  *aRowList = mRowList;
  NS_IF_ADDREF(*aRowList);

  return NS_OK;
}

NS_IMETHODIMP
nsDatabaseOutlinerView::SetRowList(nsIDatabaseRowList* aRowList)
{
  if (mRowList) {
    mRowList->RemoveObserver(this);
  }

  mRowList = aRowList;

  if (mRowList)
    mRowList->AddObserver(this);

  return NS_OK;
}


NS_IMETHODIMP
nsDatabaseOutlinerView::OnAppendRow(nsIDatabaseRow *aRow)
{
#ifdef DEBUG
  printf("nsDatabaseOutlinerView::OnAppendRow\n");
#endif
  PRInt32 count;
  GetRowCount(&count);
  mBoxObject->RowCountChanged(count, 1);

  return NS_OK;
}
    
NS_IMETHODIMP
nsDatabaseOutlinerView::OnInsertRow(PRInt32 aRowIndex, nsIDatabaseRow* aRow)
{
#ifdef DEBUG
  printf("nsDatabaseOutlinerView::OnInsertRow\n");
#endif
  mBoxObject->RowCountChanged(aRowIndex, 1);

  return NS_OK;
}
    
NS_IMETHODIMP
nsDatabaseOutlinerView::OnReplaceRow(PRInt32 aRowIndex, nsIDatabaseRow* aRow)
{
#ifdef DEBUG
  printf("nsDatabaseOutlinerView::OnReplaceRow\n");
#endif
  mBoxObject->InvalidateRow(aRowIndex);

  return NS_OK;
}

NS_IMETHODIMP
nsDatabaseOutlinerView::OnRemoveRow(PRInt32 aRowIndex)
{
#ifdef DEBUG
  printf("nsDatabaseOutlinerView::OnRemoveRow\n");
#endif
  mBoxObject->RowCountChanged(aRowIndex, -1);

  return NS_OK;
}

NS_IMETHODIMP
nsDatabaseOutlinerView::OnClearRows()
{
#ifdef DEBUG
  printf("nsDatabaseOutlinerView::OnClearRows\n");
#endif
  PRInt32 count;
  GetRowCount(&count);
  mBoxObject->RowCountChanged(0, -count);

  return NS_OK;
}
