/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Jan Varga <varga@utcru.sk> (Original Author)
 */

#include "nsMemory.h"
#include "nsCRT.h"
#include "nsXPIDLString.h"
#include "nsDatabaseRow.h"


//#define DEBUG 1
//#define DEBUG_REFCOUNT 1


#ifdef DEBUG_REFCOUNT
static PRInt32 gRefCnt = 0;
#endif


nsDatabaseRow::nsDatabaseRow()
{
#ifdef DEBUG
  printf("nsDatabaseRow::nsDatabaseRow\n");
#endif
#ifdef DEBUG_REFCOUNT
  gRefCnt++;
#endif
  NS_INIT_ISUPPORTS();
  mColumnCount = 0;
  mValues = nsnull;
}

nsDatabaseRow::~nsDatabaseRow()
{
#ifdef DEBUG
  printf("nsDatabaseRow::~nsDatabaseRow\n");
#endif
#ifdef DEBUG_REFCOUNT
  if (--gRefCnt == 0)
    printf("Last row destroyed\n");
#endif
  if (mValues) {
    for (PRInt16 i = 0; i < mColumnCount; i++)
      if (mValues[i])
        nsMemory::Free(mValues[i]);
    nsMemory::Free(mValues);
  }
}


NS_IMPL_ISUPPORTS1(nsDatabaseRow,
                   nsIDatabaseRow);


NS_IMETHODIMP
nsDatabaseRow::GetColumnCount(PRInt16 *aColumnCount)
{
  *aColumnCount = mColumnCount;
    
  return NS_OK;
}

NS_IMETHODIMP
nsDatabaseRow::Init(PRInt16 aColumnCount)
{
  mColumnCount = aColumnCount;
  PRInt32 bytes = mColumnCount * sizeof(PRUnichar*);
  mValues = (PRUnichar**)nsMemory::Alloc(bytes);
  nsCRT::memset(mValues, 0, bytes);

  return NS_OK;
}

NS_IMETHODIMP
nsDatabaseRow::GetValue(PRInt16 aColumnIndex, PRUnichar** _retval)
{
  if (! mValues)
    return NS_ERROR_NOT_INITIALIZED;
    
  if (aColumnIndex >= 0 && aColumnIndex < mColumnCount && mValues[aColumnIndex])
    *_retval = nsXPIDLString::Copy(mValues[aColumnIndex]);
  else
    *_retval = nsnull;
        
  return NS_OK;
}

NS_IMETHODIMP nsDatabaseRow::GetValueConst(PRInt16 aColumnIndex, const PRUnichar** aValueConst)
{
  if (! mValues)
    return NS_ERROR_NOT_INITIALIZED;

  if (aColumnIndex >= 0 && aColumnIndex < mColumnCount)
    *aValueConst = mValues[aColumnIndex];
  else
    *aValueConst = nsnull;
    
  return NS_OK;
}
    
NS_IMETHODIMP
nsDatabaseRow::SetValue(PRInt16 aColumnIndex, const PRUnichar* aValue)
{
  if (! mValues)
    return NS_ERROR_NOT_INITIALIZED;

  if (aColumnIndex >= 0 && aColumnIndex < mColumnCount) {
    if (mValues[aColumnIndex])
      nsMemory::Free(mValues[aColumnIndex]);
    mValues[aColumnIndex] = nsXPIDLString::Copy(aValue);
  }
  
  return NS_OK;
}
