/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Jan Varga <varga@utcru.sk> (Original Author)
 */

#include "nsIGenericFactory.h"

#include "nsDatabaseCID.h"

#include "nsDatabaseAutoCompleteSession.h"
#include "nsDatabaseDataSource.h"
#include "nsDatabaseOutlinerView.h"
#include "nsDatabaseResource.h"
#include "nsDatabaseRow.h"
#include "nsDatabaseRowList.h"

#ifdef MOZ_ENABLE_PGSQL
#include "nsDatabaseConnectionPgsql.h"
#endif

#ifdef MOZ_ENABLE_MYSQL
#include "nsDatabaseConnectionMysql.h"
#endif


NS_GENERIC_FACTORY_CONSTRUCTOR(nsDatabaseAutoCompleteSession)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsDatabaseDataSource)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsDatabaseOutlinerView)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsDatabaseResource)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsDatabaseRow)
NS_GENERIC_FACTORY_CONSTRUCTOR(nsDatabaseRowList)

#ifdef MOZ_ENABLE_PGSQL
NS_GENERIC_FACTORY_CONSTRUCTOR(nsDatabaseConnectionPgsql)
#endif

#ifdef MOZ_ENABLE_MYSQL
NS_GENERIC_FACTORY_CONSTRUCTOR(nsDatabaseConnectionMysql)
#endif


static nsModuleComponentInfo components[] =
{
  { "Database Auto Complete Session",
    NS_DATABASE_AUTO_COMPLETE_SESSION_CID,
    NS_DATABASE_AUTO_COMPLETE_SESSION_CONTRACTID,
    nsDatabaseAutoCompleteSessionConstructor
  },
  { "Database Data Source",
    NS_DATABASE_DATA_SOURCE_CID, 
    NS_DATABASE_DATA_SOURCE_CONTRACTID,
    nsDatabaseDataSourceConstructor
  },
  { "Database Outliner View",
    NS_DATABASE_OUTLINER_VIEW_CID,
    NS_DATABASE_OUTLINER_VIEW_CONTRACTID,
    nsDatabaseOutlinerViewConstructor
  },
  { "Database Resource",
    NS_DATABASE_RESOURCE_CID,
    NS_DATABASE_RESOURCE_CONTRACTID,
    nsDatabaseResourceConstructor
  },
  { "Database Row",
    NS_DATABASE_ROW_CID,
    NS_DATABASE_ROW_CONTRACTID,
    nsDatabaseRowConstructor
  },
  { "Database Row List",
    NS_DATABASE_ROW_LIST_CID,
    NS_DATABASE_ROW_LIST_CONTRACTID,
    nsDatabaseRowListConstructor
  },

#ifdef MOZ_ENABLE_PGSQL
  { "Database Connection Pgsql",
    NS_DATABASE_CONNECTION_PGSQL_CID,
    NS_DATABASE_CONNECTION_PGSQL_CONTRACTID,
    nsDatabaseConnectionPgsqlConstructor
  },
#endif

#ifdef MOZ_ENABLE_MYSQL
  { "Database Connection Mysql",
    NS_DATABASE_CONNECTION_MYSQL_CID,
    NS_DATABASE_CONNECTION_MYSQL_CONTRACTID,
    nsDatabaseConnectionMysqlConstructor
  },
#endif
};

NS_IMPL_NSGETMODULE("nsDatabaseModule", components)
