/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Jan Varga <varga@utcru.sk> (Original Author)
 */

#ifndef nsDatabaseConnectionPgsql_h
#define nsDatabaseConnectionPgsql_h                                                  


#include "libpq-fe.h"

#include "prcvar.h"
#include "nsString.h"
#include "nsISupportsArray.h"
#include "nsIDOMWindowInternal.h"
#include "nsIThread.h"
#include "nsIRunnable.h"
#include "nsIProxyObjectManager.h"
#include "nsProxiedService.h"

#include "nsIDatabaseConnection.h"
#include "nsIDatabaseConnectionPgsql.h"


#define MSG_BEGIN_EXECUTION    0
#define MSG_END_EXECUTION    1


class nsDatabaseConnectionPgsql : public nsIRunnable,
                                  public nsIDatabaseConnection,
                                  public nsIDatabaseConnectionPgsql
{
  public:
    nsDatabaseConnectionPgsql();
    virtual ~nsDatabaseConnectionPgsql();

    NS_DECL_ISUPPORTS

    NS_DECL_NSIRUNNABLE

    NS_DECL_NSIDATABASECONNECTION

    NS_DECL_NSIDATABASECONNECTIONPGSQL

  protected:
    nsresult BuildRows(PGresult* aResult, nsIDatabaseRowList** _retval);
    nsresult NotifyObservers(char aMessageType, PRBool aUseProxy);

  private:
    PRInt32                                 mStatusCode;
    nsCString                               mStatusMessage;
    nsCOMPtr<nsIDOMWindowInternal>          mWindow;
    PGconn*                                 mConnection;
    PRBool                                  mConnected;
    PRLock*                                 mLock;
    PRCondVar*                              mCondVar;
    PRBool                                  mRunning;
    PRBool                                  mCancelled;
    nsCOMPtr<nsIProxyObjectManager>         mProxyObjectManager;
    nsCOMPtr<nsIThread>                     mThread;
    nsString                                mQuery;
    nsCOMPtr<nsIDatabaseConnectionListener> mListener;
    nsCOMPtr<nsISupports>                   mContext;
    nsCOMPtr<nsISupportsArray>              mObservers;
};


#endif // nsDatabaseConnectionPgsql_h
